% DYNAMIC CORRELATION ESTIMATION VIA WINDOWLESS HEAT KERNEL METHOD
%
%Simulation of accuracy of state space estimation using sliding window (SW), 
% tapered sliding window (TSW) and the proposed windowless (heat kernel)
% methods, which was published in
% 
% Huang, S.-G., Chung, M.K., Carroll, I.C., Goldsmith, H.H. 2019 Dynamic 
% functional connectivity using heat kernel. IEEE Data Science Workshop (DSW)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.DSW.pdf
%
% The simulation below will generate Figure 4 in the paper. This code is
% downloaded from  http://brainimaging.waisman. wisc.edu/~chung/circle.
%
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
% Update history:
%     May  7, 2019 created by Huang
%     May 24,2019 edited by Chung
% 


T=295       % number of time points
num_seg=10; % number of time sections (assume state remains the same in each segment)
num_subj=50 % number of subjects
win=22      % FWHM of window or kenel

K=3         % number of clusters (states)
rep=10      % number of replicates in k-means clutering

iter=1      % number of independent simulations
NOISE=[0.1:0.1:1];  % range of noise sandard deviation

%% heat kernel defined on a circle
bw=5*10^-4;             % bandwidth of heat kernel
kernel=heat(T,bw,T);    % heat kernel

%% Accuracy of state space estimation for difference noise sandard deviation
AccSq=zeros(1, length(NOISE));
AccTSW=zeros(1, length(NOISE));
AccHeat=zeros(1, length(NOISE));

for sim=1:iter
    sim=sim+1
    
    %% ground truth of state spaces and simulated data
    trueState=[];
    for i=1:num_subj
        
        % ground truth of Tx1 state space of i-th subject
        truth=groundtruth(T,num_seg);
        trueState(:,i)=truth;
        
        % two Tx1 normal distributed random sequences with zero mean and unit standard deviation
        rand1=normrnd(0,1,T,1);
        rand2=normrnd(0,1,T,1);
        
        % simulated data based on the ground truth of state space
        X(:,i)=rand1;
        Y(:,i)=(2-truth).*rand1+(1-abs(2-truth)).*rand2;
        % rand1 in State 1,  rand2 in State 2,  -rand1 in State 3
        % i.e., y=x in State, y~=x in State 2,  y=-x in State 3
    end
    
    %% dynamic correlations and state space estimation
    for nn=1:length(NOISE)
        
        % Add noise with standard deviation, sigma, into Txnum_subj data X and Y
        sigma=NOISE(nn);
        Xnoise=X+ normrnd(0,sigma,T,num_subj);  
        Ynoise=Y+ normrnd(0,sigma,T,num_subj);
        
        % dynamic correlations using SW-, TSW- and heat kernel method
        SWCorr=[];
        TSWCorr=[];
        HeatCorr=[];        
        for i=1:num_subj
            % Tx1 data of i-th subject
            xn=Xnoise(:,i);
            yn=Ynoise(:,i);
            
            % Tx1 dynamic correlations of i-th subject
            SWCorr(:,i)=SW(xn,yn,win);   
            TSWCorr(:,i)=TSW(xn,yn,win);
            HeatCorr(:,i)=windowless(xn,yn,kernel);
        end
        
        % k-means clustering of T*num_subj correlations
        % SWState, TSWState and HeatState are (T*num_subj)x1 state spaces
        % SWCen, TSWCen and HeatCen are the Kx1 centroids (average correlations)
        [SWState,SWCen]=kmeans_sort(SWCorr(:),K,rep);
        [TSWState,TSWCen]=kmeans_sort(TSWCorr(:),K,rep);
        [HeatState,HeatCen]=kmeans_sort(HeatCorr(:),K,rep);
        
        
        % accuracy computed by the fraction of estimated state equals to ground truth
        AccSq(nn)=AccSq(nn)+   sum(trueState(:)==SWState)/T/num_subj ;
        AccTSW(nn)=AccTSW(nn)+ sum(trueState(:)==TSWState)/T/num_subj;
        AccHeat(nn)=AccHeat(nn)+ sum(trueState(:)==HeatState)/T/num_subj;
               
    end
    
end

%% average accuracy of simulations
AccSq=AccSq/iter
AccTSW=AccTSW/iter
AccHeat=AccHeat/iter

%% Plot accuracy results
figure
plot(NOISE,AccSq,'g-.','Linewidth',1)
hold on
plot(NOISE,AccTSW,'r--','Linewidth',1)
hold on
plot(NOISE,AccHeat,'b-','Linewidth',1)
hold off
title('Accuracy of state space estimation')
legend('Sliding window','Tapered sliding window','Heat kernel')
xlabel('Noise standard deviation \sigma')
xlim([NOISE(1) NOISE(end)])
ylim([0.75 0.95])
legend boxoff

figure_bigger(20)


