function dyncorr=SW(x,y,win)
% Computes correlation based on the sliding window method that uses a square window.
%
% x :       input data 1 of size T x p (T: number of time points, p: number of nodes)
% y :       input data 2 of size T x p
% win:      window size
% dyncorr:  dynamic correlations of size T x p
%
% The matheamtical detail is given in
% Huang, S.-G., Chung, M.K., Carroll, I.C., Goldsmith, H.H. 2019 Dynamic 
% functional connectivity using heat kernel. IEEE Data Science Workshop (DSW)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.DSW.pdf
%
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
% Update history:
%     May  7, 2019 created by Huang
%     May 24,2019 edited by Chung
% 


%%
[T,p]=size(x);  % T: number of time points, p: number of nodes

%% data connecting its mirror reflection
x2=x([1:end end end:-1:2],:);   % 2T x p
y2=y([1:end end end:-1:2],:);

%% pairwise dynamic correlation between paired columns of x2 and y2
dyncorr=zeros(T,p);
for i=1:T
    % indexes of data for i-th time point
    if mod(win,2)==1
        ind=mod( i-1-(win-1)/2:i-1+(win-1)/2 , 2*T) +1;   % odd window size
    else
        ind=mod( i-win/2:i+win/2-1 , 2*T) +1;             % even window size
    end
    
    % segmented data within window
    x2w=x2(ind,:);      % win x p
    y2w=y2(ind,:);
    
    % dynamic mean
    mux=mean(x2w,1);     % 1 x p
    muy=mean(y2w,1);
    
    % dynamic variance
    varx=mean( (x2w-mux).^2 , 1);	% 1 x p
    vary=mean( (y2w-muy).^2 , 1);
    
    % dynamic covariance
    covxy=mean( (x2w-mux).*(y2w-muy) , 1);      % 1 x p
    
    % dynamic correlation
    dyncorr(i,:)=covxy./sqrt(varx)./sqrt(vary); % 1 x p
end

