function dyncorr=TSW(x,y,win)
% Computes correlation based on the tapered sliding window (TSW) method
% TSW uses a square window convolved with a Gaussian kernel of standard deviation 3.
%
% x :       input data 1 of size T x p (T: number of time points, p: number of nodes)
% y :       input data 2 of size T x p
% win:      window size of square window, which will be convloved with Gaussian kernel.
% dyncorr:  dynamic correlations of size T x p
%
% The mathematical detail is given in
% Huang, S.-G., Chung, M.K., Carroll, I.C., Goldsmith, H.H. 2019 Dynamic 
% functional connectivity using heat kernel. IEEE Data Science Workshop (DSW)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.DSW.pdf
%
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
% Update history:
%     May  7, 2019 created by Huang
%     May 24, 2019 edited by Chung


%%
[T,p]=size(x);  % T: number of time points, p: number of nodes

%% data connecting its mirror reflection
x2=x([1:end end end:-1:2],:);   % 2T x p
y2=y([1:end end end:-1:2],:);

%% Tapered window
% Square window
win2=win+30;    % window size of trancation after Gaussian convolution
square=[zeros(1,15) ones(1,win) zeros(1,15)];

% Gaussian kernel
sigma=3;    % standard deviation
gaussian=1/sqrt(2*pi*sigma^2) * exp(-[-win2:win2].^2/sigma^2/2);

% Square window convolved with Gaussian
tapered=conv(square,gaussian,'same');

% normalized to summation being 1
tapered=tapered/sum(tapered);

%% pairwise dynamic correlation between paired columns of x2 and y2
dyncorr=zeros(T,p);
for i=1:T
    % indexes of data for i-th time point
    if mod(win2,2)==1
        ind=mod( i-1-(win2-1)/2:i-1+(win2-1)/2 , 2*T) +1;   % odd window size
    else
        ind=mod( i-win2/2:i+win2/2-1 , 2*T) +1;             % even window size
    end
    
    % segmented data within window
    x2w=x2(ind,:);      % win2 x p
    y2w=y2(ind,:);
    
    % dynamic mean
    mux=tapered*x2w;     % 1 x p
    muy=tapered*y2w;
    
    % dynamic variance
    varx=tapered*( (x2w-mux).^2 );	% 1 x p
    vary=tapered*( (y2w-muy).^2 );
    
    % dynamic covariance
    covxy=tapered*( (x2w-mux).*(y2w-muy) );      % 1 x p
    
    % dynamic correlation
    dyncorr(i,:)=covxy./sqrt(varx)./sqrt(vary); % 1 x p
end



