function kernel=heat(T,bw,deg)
% Defines heat kernel on a circle of circumference 2
% The data of size T is connected with its mirror reflection into 2T data.
%
% T:        number of time points
% bw:       bandwidth of heat kernel
% deg:      degree of cosine series representation
% kernel:   heat kernel of size T x 2T
%
% The heat kernel is used to compute dynamic correlations at T time points from the 2T data.
% The mathematical detail is given in
% Huang, S.-G., Chung, M.K., Carroll, I.C., Goldsmith, H.H. 2019 Dynamic 
% functional connectivity using heat kernel. IEEE Data Science Workshop (DSW)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.DSW.pdf
%
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
% Update history:
%     May  7, 2019 created by Huang
%     May 24, 2019 edited by Chung
% 

%% Cosine sereies with period 2 (circle with circumference 2)
% period 2
t=[0:2*T-1]'/T;

% cosinse sereis cos(pi*l*t) for l=0...deg
cosbasis=zeros(2*T,deg+1);     % 2T x (deg+1)
cosbasis(:,1)=1;
cosbasis(:,2:deg)=sqrt(2)*cos(pi*t*[1:deg-1]);
cosbasis(:,deg+1)=cos(pi*t*deg);

% Least squares estimation (LSE)
cosLSE=pinv(cosbasis'*cosbasis)*cosbasis';   % (deg+1) x 2T


%% Heat kerenl defined on the circle
% weights of heat kernel
weight = exp(-[0:deg].^2*pi^2*bw);      % 1 x (deg+1)

% heat kerenel
kernel=cosbasis(1:T,:)*diag(weight)*cosLSE;    % T x 2T

% eliminate very small negative values
kernel=kernel.*(kernel>=0);

% normalized to summation being 1
kernel=kernel./(sum(kernel,2)*ones(1,2*T));



