function dyncorr=windowless(x,y,kernel)
% Computes correlation based on windowless dynamic correlation using the heat kernel
%
% x :       input data 1 of size T x p (T: number of time points, p: number of nodes)
% y :       input data 2 of size T x p
% kernel:   kernel function of size T x 2T
% dyncorr:  dynamic correlations of size T x p
%
% The mathematical detail is given in
% Huang, S.-G., Chung, M.K., Carroll, I.C., Goldsmith, H.H. 2019 Dynamic 
% functional connectivity using heat kernel. IEEE Data Science Workshop (DSW)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.DSW.pdf
%
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
% Update history:
%     May  7, 2019 created by Huang
%     May 24, 2019 edited by Chung

%%
[T,p]=size(x);  % T: number of time points, p: number of nodes

%% data (each column) connecting its mirror reflection
x2=x([1:end end end:-1:2],:);   % 2T x p
y2=y([1:end end end:-1:2],:);

%% pairwise dynamic correlation between paired columns of x2 and y2
dyncorr=zeros(T,p);
for i=1:T
    % kernel at i-th time point
    Ki=kernel(i,:);  % 1 x 2T
    
    % dynamic mean
    mux=Ki*x2;     % 1 x p
    muy=Ki*y2;
    
    % dynamic variance
    varx=Ki*( (x2-mux).^2 );	% 1 x p
    vary=Ki*( (y2-muy).^2 );
    
    % dynamic covariance
    covxy=Ki*( (x2-mux).*(y2-muy) );      % 1 x p
    
    % dynamic correlation
    dyncorr(i,:)=covxy./sqrt(varx)./sqrt(vary); % 1 x p
end


